

set -ex



./run_qt_test.sh
test -d $PREFIX/include/qt6/QtCore
test -f $PREFIX/lib/libQt6Core${SHLIB_EXT}
test -d $PREFIX/include/qt6/QtGui
test -f $PREFIX/lib/libQt6Gui${SHLIB_EXT}
test -d $PREFIX/include/qt6/QtNetwork
test -f $PREFIX/lib/libQt6Network${SHLIB_EXT}
test -d $PREFIX/include/qt6/QtOpenGL
test -f $PREFIX/lib/libQt6OpenGL${SHLIB_EXT}
test -d $PREFIX/include/qt6/QtOpenGLWidgets
test -f $PREFIX/lib/libQt6OpenGLWidgets${SHLIB_EXT}
test -d $PREFIX/include/qt6/QtPrintSupport
test -f $PREFIX/lib/libQt6PrintSupport${SHLIB_EXT}
test -d $PREFIX/include/qt6/QtSql
test -f $PREFIX/lib/libQt6Sql${SHLIB_EXT}
test -d $PREFIX/include/qt6/QtTest
test -f $PREFIX/lib/libQt6Test${SHLIB_EXT}
test -d $PREFIX/include/qt6/QtWidgets
test -f $PREFIX/lib/libQt6Widgets${SHLIB_EXT}
test -d $PREFIX/include/qt6/QtXml
test -f $PREFIX/lib/libQt6Xml${SHLIB_EXT}
test -f $PREFIX/lib/qt6/plugins/platforms/libqxcb.so
test -f $PREFIX/lib/qt6/plugins/platforms/libqeglfs.so
test -f $PREFIX/lib/qt6/plugins/sqldrivers/libqsqlite${SHLIB_EXT}
test -f $PREFIX/lib/qt6/plugins/sqldrivers/libqsqlmysql${SHLIB_EXT}
test -f $PREFIX/lib/qt6/plugins/sqldrivers/libqsqlpsql${SHLIB_EXT}
test -f $PREFIX/lib/qt6/plugins/imageformats/libqjpeg${SHLIB_EXT}
test ! -f $PREFIX/lib/libQt6WaylandClient${SHLIB_EXT}
test ! -f $PREFIX/lib/libQt6WaylandCompositor${SHLIB_EXT}
qmake6 --version
exit 0
